<?php
/**
 * 
 * @author  AME 16 juin 2009
 */
class SpiralDi_Container_DefaultTest extends PHPUnit_Framework_TestCase {

    public function getSchema(){
        // get the schema from the file
        $xmlBuilder = new SpiralDi_SchemaBuilder_Xml();
        $xmlBuilder->setFileName(FIXTURES_PATH . '/SpiralDi/SchemaBuilder/Xml/testSchema.xml');
        return $xmlBuilder->buildSchema();
    }

    public function getContainer(){
        return new SpiralDi_Container_Default($this->getSchema());
    }
    
    public function testGetService(){        
        //let's check that the container works well
        $container = $this->getContainer();
        $buildedService_test = $container->getService('test');
        $test_injectedParams = $buildedService_test->getInjectedParams();
        
        $this->assertEquals('string', $test_injectedParams[0]);
        $this->assertEquals(23, $test_injectedParams[1]);
        $this->assertEquals($container->service1, $test_injectedParams[2]);
        $this->assertEquals('configParam', $test_injectedParams[3]);
        $this->assertEquals('with no argument', $test_injectedParams[4]);

        // check if the container itself has been injected
        $containerAware = $container->getService('containerAware');
        $this->assertEquals('SpiralDi_Container_Default', get_class($containerAware->getDiContainer()));

        // check if the container itself has been injected
        $containerAware = $container->getService('serviceWithContainer');
        $this->assertEquals('SpiralDi_Container_Default', get_class($containerAware->getDiContainer()));

        // check if the inherited service call all parent defined methods
        $inheritedService = $container->inheritedService;
        $inheritedService_injectedParams = $inheritedService->getInjectedParams();
        $this->assertEquals('string', $inheritedService_injectedParams[0]);
        $this->assertEquals(23, $inheritedService_injectedParams[1]);
        $this->assertEquals($container->service1, $inheritedService_injectedParams[2]);
        $this->assertEquals('configParam', $inheritedService_injectedParams[3]);
        $this->assertEquals('with no argument', $inheritedService_injectedParams[4]);

        // and that the new method has been called
        $this->assertEquals('injectedChildParam', $inheritedService->getChildMethodArgs());

        // check that calling an inexistant service throws an exception
        try{
            $container->brokenDependencyService;
            $this->fail('the SpiralDi_Schema_Exception_UnknownService Exception has not been throwed');
        } catch(SpiralDi_Schema_Exception_UnknownService $e){
            $this->assertTrue(true);
        }
        
        try{
            // now, inject the container with the inexistantService
            $container->setService('inexistantService', new DiTest_InexistantService);
            $container->brokenDependencyService;
            $this->assertTrue(true);
        } catch(SpiralDi_Schema_Exception_UnknownService $e){
            $this->fail('registering a new service on the fly in the container doesnt works');
        }

        $serviceFactory = $container->serviceFactory;
        $this->assertEquals('DiTest_Service1', get_class($serviceFactory));
    }

    public function testScopes(){
        $container = $this->getContainer();

        // by default, scope is singleton
        $this->assertTrue($container->test === $container->test);

        // if the scope is "prototype", objects are not strictly equals
        $this->assertFalse($container->prototype === $container->prototype);
    }
}
?>
