<?php
/**
 * 
 * @author  AME 17 juin 2009
 */
class SpiralDi_SchemaBuilder_XmlTest extends PHPUnit_Framework_TestCase {
    public function testSetFileName(){
        
    }

    public function testBuildSchema(){
        $xmlBuilder = new SpiralDi_SchemaBuilder_Xml();
        $xmlBuilder->setFileName(FIXTURES_PATH . '/SpiralDi/SchemaBuilder/Xml/testSchema.xml');
        $schema = $xmlBuilder->buildSchema();

        // now, let's go throught the schema to check if everything has going well
        //$this->assertEquals(4, count($schema->getServices()));

        // test the first service
        $testService = $schema->getService('test');
        $this->assertEquals('test', $testService->getName());
        $this->assertEquals('DiTest_Test', $testService->getClassName());
        $this->assertEquals(1, count($testService->getMethods()));

        // test if the injection method respects the schema
        $testInjectionMethod = $testService->getMethod('testInjection');
        $this->assertEquals('testInjection', $testInjectionMethod->getName());

        // count of arguments
        $this->assertEquals(5,count($testInjectionMethod->getArguments()));

        // for default arguments (string)
        $this->assertEquals('SpiralDi_Schema_Argument_Default',get_class($testInjectionMethod->getArgument(0)));
        $this->assertEquals('string', $testInjectionMethod->getArgument(0)->getValue());

        // for ints
        $this->assertEquals('SpiralDi_Schema_Argument_Default',get_class($testInjectionMethod->getArgument(1)));
        $this->assertEquals(23, $testInjectionMethod->getArgument(1)->getValue());

        // for services
        $this->assertEquals('SpiralDi_Schema_Argument_ServiceRef',get_class($testInjectionMethod->getArgument(2)));
        $this->assertEquals('service1', $testInjectionMethod->getArgument(2)->getValue());

        // for factories
        $this->assertEquals('SpiralDi_Schema_Argument_UseRef', get_class($testInjectionMethod->getArgument(3)));
        $this->assertEquals('config', $testInjectionMethod->getArgument(3)->getRef());
        $this->assertEquals('db.config', $testInjectionMethod->getArgument(3)->getValue());
        $this->assertEquals('getParam', $testInjectionMethod->getArgument(3)->getFactoryMethod());

        // test the second service
        $service1 = $schema->getService('service1');
        $this->assertEquals('service1', $service1->getName());
        $this->assertEquals('DiTest_Service1', $service1->getClassName());
        $this->assertEquals(1, count($service1->getMethods()));

        // constructor
        $service1Constructor = $service1->getMethod('__construct');
        $this->assertEquals('__construct', $service1Constructor->getName());
        $this->assertEquals(1, count($service1Constructor->getArguments()));
        $this->assertEquals('SpiralDi_Schema_Argument_Default', get_class($service1Constructor->getArgument(0)));
        $this->assertEquals('constructorParam', $service1Constructor->getArgument(0)->getValue());

        // test the third service
        $config = $schema->getService('config');
        $this->assertEquals('config', $config->getName());
        $this->assertEquals('DiTest_Config', $config->getClassName());
        $this->assertEquals(0, count($config->getMethods()));

        // Tests about the container aware service
        $containerAware = $schema->getService('containerAware');
        $this->assertEquals('containerAware', $containerAware->getName());
        $this->assertEquals('DiTest_ContainerAware', $containerAware->getClassName());
        $this->assertEquals(1, count($containerAware->getMethods()));
        $setDiContainer = $containerAware->getMethod('setDiContainer');

        $this->assertEquals('setDiContainer', $setDiContainer->getName());
        $this->assertEquals(1, count($setDiContainer->getArguments()));
        $this->assertEquals('SpiralDi_Schema_Argument_Container', get_class($setDiContainer->getArgument(0)));

        // and about a simple container injection
        $serviceWithContainer = $schema->getService('serviceWithContainer');
        $this->assertEquals('serviceWithContainer', $serviceWithContainer->getName());
        $this->assertEquals('DiTest_ServiceWithContainer', $serviceWithContainer->getClassName());
        $this->assertEquals(1, count($serviceWithContainer->getMethods()));
        $injectContainer = $serviceWithContainer->getMethod('injectContainer');

        $this->assertEquals('injectContainer', $injectContainer->getName());
        $this->assertEquals(1, count($injectContainer->getArguments()));
        $this->assertEquals('SpiralDi_Schema_Argument_Container', get_class($injectContainer->getArgument(0)));

        // testing inheritance
        $inheritedService = $schema->getService('inheritedService');
        $this->assertEquals($testInjectionMethod, $inheritedService->getMethod('testInjection'));
        $this->assertEquals('injectedChildParam', $inheritedService->getMethod('childMethod')->getArgument(0)->getValue());

        // testing the broken dependency
        $brokenService = $schema->getService('brokenDependencyService');
        $this->assertEquals('DiTest_BrokenDependencyService', $brokenService->getClassName());
        $this->assertEquals('SpiralDi_Schema_Argument_ServiceRef', get_class($brokenService->getMethod('__construct')->getArgument(0)));
        $this->assertEquals('inexistantService', $brokenService->getMethod('__construct')->getArgument(0)->getValue());

        // testing the service factory
        $serviceFactory = $schema->getService('serviceFactory');
        $this->assertEquals('SpiralDi_Schema_Service_Factory', get_class($serviceFactory));

        // testing scope
        $this->assertTrue($testService->isSingleton());
        $this->assertFalse($schema->getService("prototype")->isSingleton());
    }
}
?>
